/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
import gov.sandia.gnem.slbmjni.GridWeight;
import gov.sandia.gnem.slbmjni.SLBMException;
import gov.sandia.gnem.slbmjni.SlbmInterface;
import java.io.BufferedReader;

import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Map;
import java.io.*;

/**
 *
 * @author myers30
 */
public class GetTT {
    SlbmInterface slbm;
    public GetTT() {
        // Load the c++ SLBM library libslbm.so into memory.
          System.loadLibrary("slbmjni");
         // instantiate an SLBMInterface object
         slbm = new SlbmInterface();
    }
    
    public void loadModel (String modelPath) {
        slbm.loadVelocityModelBinary(modelPath);
    }
    
    public void setPath(double elat, double elon, double edepth, 
                        double slat, double slon, double sdepth,
                        String phase) throws SLBMException {
        double DEG_TO_RAD = Math.toRadians(1.);
        slbm.createGreatCircle(phase,elat*DEG_TO_RAD, 
                elon*DEG_TO_RAD, 
                edepth,
                slat*DEG_TO_RAD, 
                slon*DEG_TO_RAD, 
                sdepth);
	 // GridWeight wgt = slbm.getWeightsSource();
	 // System.out.println(wgt.toString());
    }
    
    public double quickTT(double elat, double elon, double edepth, 
                        double slat, double slon, double sdepth,
                        String phase) throws SLBMException {
        double DEG_TO_RAD = Math.toRadians(1.);
        slbm.createGreatCircle(phase,elat*DEG_TO_RAD, 
                elon*DEG_TO_RAD, 
                edepth,
                slat*DEG_TO_RAD, 
                slon*DEG_TO_RAD, 
                sdepth);

	
        return(this.slbm.getTravelTime());
    }
         
    
    
    public static void main(String[] args) {
        if (args.length != 1 && args.length !=7 && args.length!=8) {
            System.err.println ("Usage: GetTT <file>");
            System.err.println ("or");
            System.err.println ("       GetTT elat elon edepth slat slon selev phase model_path");
            System.err.println ("          model_path optional. if model_path is not inluded, then use environment variable RSTT_MODEL_CURRENT");
            System.err.println ("\n     If using fileinput then the first line of the file lists the model_path");
            System.err.println ("\n     Following lines have columns  elat elon edepth slat slon selev phase");
            System.exit(1);
        
       }
       GetTT tt = new GetTT();
       
       //tt.loadModel("/Users/Myers30/Documents/Location/SLBMWorkingGroup/ModelValidation/rstt20140326");
       //tt.loadModel("/Users/Myers30/Documents/Location/SLBMWorkingGroup/ModelValidation/rstt20140311");
       //tt.loadModel("/Users/Myers30/Documents/Location/SLBMWorkingGroup/Models/rstt_global_201305_validation");
       //tt.loadModel("/Users/Myers30/Documents/Location/SLBMWorkingGroup/ModelValidation/rstt20140212");
       //tt.loadModel("/Users/Myers30/Documents/Location/SLBMWorkingGroup/ModelValidation/PNE_fits/rift4s");
        if (args.length == 7 || args.length == 8) {
            if (args.length == 7 ) {
                // tt.loadModel("/Users/myers30/Documents/Location/SLBMWorkingGroup/ModelValidation/RSTT_Model_Release_April_2014");
//		tt.loadModel(Misc.MyEnvironment.getEnvVar("RSTT_MODEL_CURRENT"));
                Map env = (Map) System.getenv();
                String value = (String) env.get("RSTT_MODEL_CURRENT");
                if (value == null) {
			System.err.println("Environment variable RSTT_MODEL_CURRENT not set."); 
                        System.err.println("Setting model to ./rstt201404.geotess");
                        value = "./rstt201404.geotess";
		}
		tt.loadModel(value);

            } else {
                tt.loadModel(args[7]);
            }
            String model = args[0];
            double elat = Double.parseDouble(args[0]);
            double elon = Double.parseDouble(args[1]);
            double edepth = Double.parseDouble(args[2]);
            double slat = Double.parseDouble(args[3]);
            double slon = Double.parseDouble(args[4]);
            double selev = Double.parseDouble(args[5]);
            String phase = args[6];
            
            try {
                tt.setPath(elat, elon, edepth, slat, slon, -selev, phase);
                double time = tt.slbm.getTravelTime();
			
                System.out.printf("%8.3f\n",time);
            } catch (SLBMException ex) {
                Logger.getLogger(GetTT.class.getName()).log(Level.SEVERE, null, ex);
            }            
        } else {
            try {
                // BufferedReader in = IOUtilities.FileIO.openFile(args[0]);
                BufferedReader in = new BufferedReader(new FileReader(new FileDescriptor()));
                in = new BufferedReader(new FileReader(args[0]));
              
                String line = in.readLine();
                if (line.length() == 0 ) {
                // tt.loadModel("/Users/myers30/Documents/Location/SLBMWorkingGroup/ModelValidation/RSTT_Model_Release_April_2014");
		//tt.loadModel(Misc.MyEnvironment.getEnvVar("RSTT_MODEL_CURRENT"));
                Map env = (Map) System.getenv();
                String value = (String) env.get("RSTT_MODEL_CURRENT");
                if (value == null) {
                        System.err.println("Environment variable RSTT_MODEL_CURRENT not set.");
                        System.err.println("Setting model to ./rstt201404.geotess");
                        value = "./rstt201404.geotess";
                }
                tt.loadModel(value);
            } else {
                tt.loadModel(args[7]);
                }
                
                tt.loadModel(line.trim());
                double elat,elon,edepth,slat,slon,selev,ttimeObs,ttimeObsUnct,ttimePred,ttimePredUnct;
		int evid, orid, arid;
                String sta,phase;
                StringTokenizer t;
                while ((line=in.readLine()) != null) {
                      t = new StringTokenizer(line);
		      if (t.countTokens() == 18) { // RSTT validation data format
		        evid = Integer.parseInt(t.nextToken());	
		        orid = Integer.parseInt(t.nextToken());	
		        arid = Integer.parseInt(t.nextToken());	
			sta = t.nextToken();
                        phase = t.nextToken();
                        ttimeObs = Double.parseDouble(t.nextToken());
                        ttimeObsUnct = Double.parseDouble(t.nextToken());
                        elat = Double.parseDouble(t.nextToken());
                        elon = Double.parseDouble(t.nextToken());
                        edepth = Double.parseDouble(t.nextToken());
                        slat = Double.parseDouble(t.nextToken());
                        slon = Double.parseDouble(t.nextToken());
                        selev = Double.parseDouble(t.nextToken());
            try {
                        tt.setPath(elat, elon, edepth, slat, slon, -selev, phase);
                        ttimePred = tt.slbm.getTravelTime();
                      System.out.printf("%s %8.3f %8.3f\n",line,ttimePred,ttimeObs-ttimePred);
	    } catch (SLBMException ex) {
		   Logger.getLogger(GetTT.class.getName()).log(Level.SEVERE, null, ex); 
	    }
		      } else {   // generic input
                         elat = Double.parseDouble(t.nextToken());
                         elon = Double.parseDouble(t.nextToken());
                         edepth = Double.parseDouble(t.nextToken());
                         slat = Double.parseDouble(t.nextToken());
                         slon = Double.parseDouble(t.nextToken());
                         selev = Double.parseDouble(t.nextToken());
                         phase = t.nextToken();
	    try {
                         tt.setPath(elat, elon, edepth, slat, slon, -selev, phase);
                         ttimePred = tt.slbm.getTravelTime();
                         System.out.printf("%8.3f %s\n",ttimePred,line);
            } catch (SLBMException ex) {
                //ttimePred = Double.parseDouble(t.nextToken());
                //System.out.printf("%8.3f %s\n",ttimePred,line);
		   Logger.getLogger(GetTT.class.getName()).log(Level.SEVERE, null, ex); 
	    }
		      }
                }
            } catch (Exception ex) {
                Logger.getLogger(GetTT.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
}

